@@ -3,8 +3,8 @@ from hashlib import sha1
 import logging
 import inspect
 
-from pulsar.utils.py2py3 import iteritems
-from pulsar.utils.tools import gen_unique_id
+from pulsar.utils.httpurl import iteritems
+from pulsar.utils.security import gen_unique_id
 
 
 __all__ = ['JobMetaClass','Job','PeriodicJob',
@@ -92,53 +92,53 @@ JobBase = JobMetaClass('JobBase',(object,),{'abstract':True})
 
 class Job(JobBase):
     '''The Job class which is used in a distributed task queue.
-    
+
 .. attribute:: name
 
     The unique name which defines the Job and which can be used to retrieve
     it from the job registry.
-    
+
 .. attribute:: abstract
 
     If set to ``True`` (default is ``False``), the Job won't be registered
     with the :class:`JobRegistry`.
-    
+
 .. attribute:: autoregister
 
     If ``False`` (default is ``True``), the Job need to be registered
     manually with the Job registry.
-    
+
 .. attribute:: type
 
     Type of Job, one of ``regular`` and ``periodic``.
-    
+
 .. attribute:: timeout
 
     An instance of a datetime.timedelta or ``None``. If set, it represents the
     time lag after which a task which did not start expires.
-    
+
     Default: ``None``.
-    
+
 .. attribute:: can_overlap
 
     Boolean indicating if this job can generate overlapping tasks. It can
     also be a callable which accept the same input parameters as the job
     callable function.
-    
+
     Default: ``True``.
-    
+
 .. attribute:: doc_syntax
 
     The doc string syntax.
-    
+
     Default: ``markdown``
-    
+
 .. attribute:: loglevel
 
     Level of task logging
-    
+
     Default: ``None``
-    
+
 .. attribute:: logger
 
     an instance of a logger.
@@ -153,12 +153,12 @@ class Job(JobBase):
     doc_syntax = 'markdown'
     can_overlap = True
     _ack = True
-        
+
     def __call__(self, consumer, *args, **kwargs):
         '''The Jobs' task executed by the consumer. This function needs to be
 implemented by subclasses.'''
         raise NotImplementedError("Jobs must define the run method.")
-    
+
     def make_task_id(self, args, kwargs):
         '''Get the task unique identifier.
 This can be overridden by Job implementation.
@@ -167,7 +167,7 @@ This can be overridden by Job implementation.
 :parameter kwargs: dictionary of key-valued parameters passed to
     the job callable.
 :rtype: a native string.
-    
+
 Called by the :attr:`TaskQueue.scheduler` when creating a new task.
 '''
         can_overlap = self.can_overlap
@@ -178,44 +178,47 @@ Called by the :attr:`TaskQueue.scheduler` when creating a new task.
         else:
             suffix = ''
             if args:
-                suffix = 'args(' + ', '.join((str(a) for a in args)) + ')'
+                suffix = ' args(%s)' % ', '.join((str(a) for a in args))
             if kwargs:
-                suffix += 'kwargs(' + ', '.join('{0}= {1}'.\
-                            format(k,kwargs[k]) for k in sorted(kwargs)) + ')'
-            name = self.name + suffix
+                suffix += ' kwargs(%s)' % ', '.join(('%s=%s' % (k, kwargs[k])\
+                            for k in sorted(kwargs)))
+            name = '%s%s' % (self.name, suffix)
             return sha1(name.encode('utf-8')).hexdigest()[:8]
-    
+
     def on_same_id(self, task):
         '''Callback invocked when a task has an id equal to a task already
 submitted to the task queue. By default return None and the
 task is aborted.'''
         return None
-        
+
     def ack(self, args, kwargs):
         '''Return ``True`` if a Job task will acknowledge the task queue
  once the result is ready or an error has occured.
  By default it returns ``True`` but it can be overridden so that its
  behaviour can change at runtime.'''
         return self._ack
-    
-    def send_to_queue(self, consumer, jobname, ack = True, **kwargs):
+
+    def send_to_queue(self, consumer, jobname, *args, **kwargs):
         '''Send a new task request to the :class:`TaskQueue`
 from within another :class:`Task`.
-This allow for tasks acting as tasks factories.
+This allows tasks to act as tasks factories.
 
 :parameter consumer: The :class:`TaskConsumer` handling the :class:`Task`.
 :parameter jobname: The name of the :class:`Job` to run.
-:parameter ack: if acknowledgment is needed. Default ``True``.
-:parameter kwargs: key-valued parameters for the task.
-:rtype: an :class:`pulsar.ActorMessage`.
+:parameter args: positional argument for the :ref:`job callable <job-callable>`.
+:parameter kwargs: key-valued parameters for the
+    :ref:`job callable <job-callable>`.
+:rtype: A :class:`pulsar.ActorMessage` if **ack=True** was passed in the
+    key-valued parameters, otherwise nothing.
 '''
         worker = consumer.worker
+        ack = kwargs.pop('ack', False)
         oper = "addtask" if ack else "addtask_noack"
-        from_task = consumer.task.id
-        return worker.monitor.send(worker, oper, jobname,
-                                   {'from_task':from_task},
-                                   **kwargs)
-    
+        res = worker.send(worker.monitor, oper, jobname,
+                          {'from_task': consumer.task.id},
+                           *args, **kwargs)
+        return res if ack else None
+
 
 class PeriodicJob(Job):
     '''A periodic :class:`Job` implementation.'''
@@ -226,13 +229,13 @@ class PeriodicJob(Job):
 It controls when the periodic Job is run.'''
     run_every = None
     '''Periodicity as a :class:`datetime.timedelta` instance.'''
-    
+
     def __init__(self, run_every = None):
         self.run_every = run_every or self.run_every
         if self.run_every is None:
             raise NotImplementedError('Periodic Jobs must have\
  a run_every attribute set, "{0}" does not have one'.format(self.name))
-   
+
     def is_due(self, last_run_at):
         """Returns tuple of two items ``(is_due, next_time_to_run)``,
 where next time to run is in seconds. e.g.
@@ -245,12 +248,12 @@ where next time to run is in seconds. e.g.
 You can override this to decide the interval at runtime.
         """
         return self.run_every.is_due(last_run_at)
-    
+
 
 
 def anchorDate(hour = 0, minute = 0, second = 0):
     td = date.today()
     return datetime(year = td.year, month = td.month, day = td.day,
                     hour = hour, minute = minute, second = second)
-    
+
 
