@@ -1,10 +1,9 @@
-'''\
-Configuration utilities. originally from gunicorn_,
+'''Configuration utilities. originally from gunicorn_,
 adapted and modified for pulsar.
 
 Original Gunicorn Licence
 
-This file is part of gunicorn released under the MIT license. 
+This file is part of gunicorn released under the MIT license.
 See the NOTICE for more information.
 
 .. _gunicorn: http://gunicorn.org/
@@ -17,40 +16,58 @@ import textwrap
 import types
 
 from pulsar import __version__, SERVER_NAME
-from pulsar.utils import system
-from pulsar.utils.py2py3 import *
+from . import system
+from .httpurl import is_string_or_native_string, to_string, to_bytes,\
+                        iteritems, HttpParser as PyHttpParser
+from .importer import import_system_file
 
 
 __all__ = ['Config',
+           'SimpleSetting',
            'Setting',
+           'defaults',
            'ordered_settings',
            'validate_string',
            'validate_callable',
            'validate_bool',
            'validate_list',
            'validate_pos_int',
-           'make_settings',
-           'make_options']
+           'validate_pos_float',
+           'make_settings']
 
-    
+class DefaultSettings:
+
+    def __init__(self):
+        # port for serving a socket
+        self.PORT = 8060
+        # timeout for asynchronous Input/Output
+        self.IO_TIMEOUT = None
+        # Maximum number of concurrenct clients
+        self.BACKLOG = 2048
+        # Actors timeout
+        self.TIMEOUT = 30
+        # mailbox clients blocking
+        self.message_timeout = 3
+
+defaults = DefaultSettings()
 KNOWN_SETTINGS = {}
 KNOWN_SETTINGS_ORDER = []
 
 
 def def_start_server(server):
     pass
-    
+
 
 def def_pre_exec(server):
     pass
-    
-    
+
+
 def default_process(worker):
     pass
 
 
 def def_pre_request(worker, req):
-    worker.log.debug("%s %s" % (req.method, req.path))
+    pass
 
 
 def def_post_request(worker, req):
@@ -59,32 +76,32 @@ def def_post_request(worker, req):
 
 def def_worker_exit(worker):
     pass
-    
-    
+
+
 def wrap_method(func):
     def _wrapped(instance, *args, **kwargs):
         return func(*args, **kwargs)
     return _wrapped
 
-    
+
 def ordered_settings():
     for name in KNOWN_SETTINGS_ORDER:
         yield KNOWN_SETTINGS[name]
-        
-        
-def make_settings(app = None, include=None, exclude=None):
-    '''Creates a dictionary of available settings for a given
-application *app*.
 
-:parameter app: Optional application name.
+def make_settings(apps=None, include=None, exclude=None):
+    '''Creates a dictionary of available settings for given
+applications *apps*.
+
+:parameter apps: Optional list of application names.
 :parameter include: Optional list of settings to include.
-:parameter app: Optional list of settings to exclude.
+:parameter exclude: Optional list of settings to exclude.
 :rtype: dictionary of :class:`pulsar.Setting` instances.'''
     settings = {}
     exclude = exclude or ()
+    apps = set(apps or ())
     for s in ordered_settings():
         setting = s()
-        if setting.app and setting.app != app:
+        if setting.app and setting.app not in apps:
             continue
         if include and setting.name not in include and not setting.app:
             continue
@@ -94,38 +111,6 @@ application *app*.
     return settings
 
 
-def make_options():
-    g_settings = make_settings(exclude=('version',))
-
-    keys = g_settings.keys()
-    def sorter(k):
-        return (g_settings[k].section, g_settings[k].order)
-
-    opts = []
-    
-    for k in keys:
-        setting = g_settings[k]
-        if not setting.flags:
-            continue
-
-        args = tuple(setting.flags)
-
-        kwargs = {
-            "dest": setting.name,
-            "metavar": setting.meta or None,
-            "action": setting.action or "store",
-            "type": setting.type or "string",
-            "default": None,
-            "help": "%s [%s]" % (setting.short, setting.default)
-        }
-        if kwargs["action"] != "store":
-            kwargs.pop("type")
-
-        opts.append(optparse.make_option(*args, **kwargs))
-
-    return tuple(opts)
-
-
 class Config(object):
     '''Dictionary containing :class:`Setting` parameters for
 fine tuning pulsar servers. It provides easy access to :attr:`Setting.value`
@@ -136,33 +121,48 @@ attribute by exposing the :attr:`Setting.name` as attribute.
     Dictionary of all :class:`Settings` instances available. The
     keys are given by the :attr:`Setting.name` attribute.
 '''
-    def __init__(self, description = None, epilog = None,
-                 version = None, app = None, include=None,
-                 exclude = None):
-        self.settings = make_settings(app,include,exclude)
+    exclude_from_config = set(('config',))
+    
+    def __init__(self, description=None, epilog=None,
+                 version=None, app=None, include=None,
+                 exclude=None):
+        self.settings = make_settings(app, include, exclude)
         self.description = description or 'Pulsar server'
         self.epilog = epilog or 'Have fun!'
         self.version = version or __version__
-        
+
+    def __iter__(self):
+        return iter(self.settings)
+    
+    def __len__(self):
+        return len(self.settings)
+    
+    def items(self):
+        for k, setting in iteritems(self.settings):
+            yield k, setting.value
+    
     def __getstate__(self):
         return self.__dict__.copy()
-    
+
     def __setstate__(self, state):
         for k,v in state.items():
             self.__dict__[k] = v
-    
+
     def __getattr__(self, name):
-        if name not in self.settings:
-            if name in KNOWN_SETTINGS:
-                return None
-            raise AttributeError("No configuration setting for: %s" % name)
-        return self.settings[name].get()
-    
+        return self.get(name)
+
     def __setattr__(self, name, value):
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
         super(Config, self).__setattr__(name, value)
-    
+
+    def get(self, name, default=None):
+        if name not in self.settings:
+            if name in KNOWN_SETTINGS:
+                return default
+            raise AttributeError("No configuration setting for: %s" % name)
+        return self.settings[name].get()
+
     def set(self, name, value):
         if name not in self.settings:
             raise AttributeError("No configuration setting for: %s" % name)
@@ -173,7 +173,7 @@ attribute by exposing the :attr:`Setting.name` as attribute.
 settings via the :meth:`Setting.add_argument`.
 
 :rtype: an instance of :class:`argparse.ArgumentParser`.
-        
+
 .. _argparser: http://docs.python.org/dev/library/argparse.html
 '''
         kwargs = {
@@ -186,14 +186,37 @@ settings via the :meth:`Setting.add_argument`.
                             version = self.version)
         setts = self.settings
         sorter = lambda x: (setts[x].section, setts[x].order)
-        for k in sorted(setts,key=sorter):
+        for k in sorted(setts, key=sorter):
             setts[k].add_argument(parser)
         return parser
 
+    def import_from_module(self, mod=None):
+        if mod:
+            self.set('config', mod)
+        try:
+            mod = import_system_file(self.config)
+        except Exception as e:
+            raise RuntimeError('Failed to read config file "%s". %s' %\
+                               (self.config, e))
+        unknowns = []
+        if mod:
+            for k in dir(mod):
+                # Skip private functions and attributes
+                kl = k.lower()
+                if k.startswith('_') or kl in self.exclude_from_config:
+                    continue
+                val = getattr(mod, k)
+                # add unknown names to list
+                if kl not in self.settings:
+                    unknowns.append((k, val))
+                else:
+                    self.set(kl, val)
+        return unknowns
+        
     def on_start(self):
         for sett in self.settings.values():
             sett.on_start()
-    
+
     @property
     def workers(self):
         return self.settings['workers'].get()
@@ -202,20 +225,20 @@ settings via the :meth:`Setting.add_argument`.
     def address(self):
         bind = self.settings['bind']
         if bind:
-            return system.parse_address(to_bytestring(bind.get()))
-        
+            return system.parse_address(to_bytes(bind.get()))
+
     @property
     def uid(self):
         user = self.settings.get('user')
         if user:
             return system.get_uid(user.get())
-        
+
     @property
     def gid(self):
         group = self.settings.get('group')
         if group:
             return system.get_gid(group.get())
-        
+
     @property
     def proc_name(self):
         pn = self.settings.get('proc_name')
@@ -227,21 +250,19 @@ settings via the :meth:`Setting.add_argument`.
             pn = self.settings.get('default_proc_name')
             if pn:
                 return pn.get()
-            
-            
+
+
 class SettingMeta(type):
     '''A metaclass which collects all setting classes and put them
 in the global ``KNOWN_SETTINGS`` list.'''
     def __new__(cls, name, bases, attrs):
         super_new = super(SettingMeta, cls).__new__
         parents = [b for b in bases if isinstance(b, SettingMeta)]
-        if not parents or attrs.pop('virtual',False):
-            return super_new(cls, name, bases, attrs)            
-    
-        attrs["order"] = len(KNOWN_SETTINGS)
         val = attrs.get("validator")
         attrs["validator"] = wrap_method(val) if val else None
-        
+        if not parents or attrs.pop('virtual',False):
+            return super_new(cls, name, bases, attrs)
+        attrs["order"] = len(KNOWN_SETTINGS)
         new_class = super_new(cls, name, bases, attrs)
         new_class.fmt_desc(attrs['desc'] or '')
         if not new_class.name:
@@ -259,32 +280,59 @@ in the global ``KNOWN_SETTINGS`` list.'''
         setattr(cls, "desc", desc)
         lines = desc.splitlines()
         setattr(cls, "short", '' if not lines else lines[0])
-        
-        
+
+
+class SettingBase(object):
+    name = None
+    '''The unique name used to access this setting.'''
+    validator = None
+    '''Validator for this setting. It provided it must be a fucntion
+accepting one positional argument, the value to validate.'''
+    value = None
+    '''The actual value for this setting.'''
+    def __str__(self):
+        return '{0} ({1})'.format(self.name, self.value)
+    __repr__ = __str__
+
+    def set(self, val):
+        '''Set *val* as the value in this :class:`Setting`.'''
+        if hasattr(self.validator,'__call__'):
+            val = self.validator(val)
+        self.value = val
+
+    def get(self):
+        return self.value
+
+    def on_start(self):
+        '''Called when pulsar server starts. It can be used to perform
+custom initialization for this :class:`Setting`.'''
+        pass
+
+
+class SimpleSetting(SettingBase):
+
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+
 # This works for Python 2 and Python 3
-class Setting(SettingMeta('BaseSettings', (object,), {'virtual': True})):
+class Setting(SettingMeta('BaseSettings', (SettingBase,), {'virtual': True})):
     '''A configuration parameter for pulsar. Parameters can be specified
 on the command line or on a config file.'''
     virtual = True
     '''If set to ``True`` the settings won't be loaded and it can be only used
 as base class for other settings.'''
-    name = None
-    '''The unique name used to access this setting.'''
     nargs = None
     '''For positional arguments. Same usage as argparse.'''
     app = None
     '''Setting for a specific :class:`Application`.'''
-    value = None
-    '''The actual value for this setting.'''
     section = None
     '''Setting section, used for creating documentation.'''
     flags = None
     '''List of options strings, e.g. ``[-f, --foo]``.'''
     choices = None
     '''Restrict the argument to the choices provided.'''
-    validator = None
-    '''Validator for this setting. It provided it must be a fucntion
-accepting one positional argument, the value to validate.'''
     type = None
     meta = None
     action = None
@@ -292,29 +340,33 @@ accepting one positional argument, the value to validate.'''
     '''Default value'''
     short = None
     desc = None
-    
-    def __init__(self):
+
+    def __init__(self, name=None, flags=None, action=None, default=None,
+                 nargs=None, description=None):
+        self.default = default if default is not None else self.default
         if self.default is not None:
             self.set(self.default)
+        self.name = name or self.name
+        self.flags = flags or self.flags
+        self.action = action or self.action
+        self.nargs = nargs or self.nargs
+        self.desc = description or self.desc
         self.short = self.short or self.desc
         self.desc = self.desc or self.short
         if self.app and not self.section:
             self.section = self.app
-    
+        if not self.section:
+            self.section = 'unknown'
+
     def __getstate__(self):
         d = self.__dict__.copy()
         return d
-    
-    def __str__(self):
-        return '{0} ({1})'.format(self.name,self.value)
-    __repr__ = __str__
-        
+
     def add_argument(self, parser):
         '''Add itself to the argparser.'''
         kwargs = {'nargs':self.nargs}
         if self.type and self.type != 'string':
             kwargs["type"] = self.type
-            
         if self.flags:
             args = tuple(self.flags)
             kwargs.update({"dest": self.name,
@@ -331,22 +383,11 @@ accepting one positional argument, the value to validate.'''
         else:
             # Not added to argparser
             return
-        
+
         parser.add_argument(*args, **kwargs)
-    
+
     def copy(self):
         return copy.copy(self)
-    
-    def get(self):
-        return self.value
-    
-    def set(self, val):
-        if hasattr(self.validator,'__call__'):
-            val = self.validator(val)
-        self.value = val
-        
-    def on_start(self):
-        pass
 
 
 def validate_bool(val):
@@ -361,9 +402,8 @@ def validate_bool(val):
     else:
         raise ValueError("Invalid boolean: %s" % val)
 
-
 def validate_pos_int(val):
-    if not isinstance(val,int_type):
+    if not isinstance(val, int):
         val = int(val, 0)
     else:
         # Booleans are ints!
@@ -372,11 +412,16 @@ def validate_pos_int(val):
         raise ValueError("Value must be positive: %s" % val)
     return val
 
+def validate_pos_float(val):
+    val = float(val)
+    if val < 0:
+        raise ValueError("Value must be positive: %s" % val)
+    return val
 
 def validate_string(val):
     if val is None:
         return None
-    if not is_bytes_or_string(val):
+    if not is_string_or_native_string(val):
         raise TypeError("Not a string: %s" % val)
     return to_string(val).strip()
 
@@ -418,11 +463,11 @@ class ConfigFile(Setting):
     default = 'config.py'
     desc = """\
         The path to a Pulsar config file.
-        
+
         Only has an effect when specified on the command line or as part of an
-        application specific configuration.    
+        application specific configuration.
         """
-        
+
 
 class Workers(Setting):
     name = "workers"
@@ -433,7 +478,7 @@ class Workers(Setting):
     default = 1
     desc = """\
         The number of worker process for handling requests.
-        
+
 If you are using a multi-process concurrency, a number in the
 the 2-4 x $(NUM_CORES) range should be good. If you are using threads this
 number can be higher."""
@@ -447,7 +492,7 @@ class Concurrency(Setting):
     desc = """\
         The type of concurrency to use: ``process`` or ``thread``.
         """
-        
+
 
 class MaxRequests(Setting):
     name = "max_requests"
@@ -458,15 +503,32 @@ class MaxRequests(Setting):
     default = 0
     desc = """\
         The maximum number of requests a worker will process before restarting.
-        
+
         Any value greater than zero will limit the number of requests a work
         will process before automatically restarting. This is a simple method
         to help limit the damage of memory leaks.
-        
+
         If this is set to zero (the default) then the automatic worker
         restarts are disabled.
         """
 
+class Backlog(Setting):
+    name = "backlog"
+    section = "Worker Processes"
+    flags = ["--backlog"]
+    validator = validate_pos_int
+    type = int
+    default = 2048
+    desc = """\
+        The maximum number of concurrent requests.
+        This refers to the number of clients that can be waiting to be served.
+        Exceeding this number results in the client getting an error when
+        attempting to connect. It should only affect servers under significant
+        load.
+
+        Must be a positive integer. Generally set in the 64-2048 range.
+        """
+
 
 class Timeout(Setting):
     name = "timeout"
@@ -477,7 +539,7 @@ class Timeout(Setting):
     default = 30
     desc = """\
         Workers silent for more than this many seconds are killed and restarted.
-        
+
         Generally set to thirty seconds. Only set this noticeably higher if
         you're sure of the repercussions for sync workers. For the non sync
         workers it just means that the worker process is still communicating and
@@ -485,15 +547,32 @@ class Timeout(Setting):
         """
 
 
+class MessageTimeout(Setting):
+    name = "message_timeout"
+    section = "Worker Processes"
+    flags = ["--message_timeout"]
+    type = int
+    default = defaults.message_timeout
+    desc = """\
+        Timeout in seconds for communication with pulsar actors.
+        A negative value corresponds to No timeout (blocking sockets) while
+        a value of 0 corresponds to non-blocking sockets."""
+
+
 class HttpProxyServer(Setting):
     name = "http_proxy"
     section = "Http"
     flags = ["--http-proxy"]
     default = ''
     desc = """\
-        The HTTP proxy server to use with HttpClient.    
+        The HTTP proxy server to use with HttpClient.
         """
-        
+    def on_start(self):
+        if self.value:
+            os.environ['http_proxy'] = self.value
+            os.environ['https_proxy'] = self.value
+
+
 class HttpClient(Setting):
     name = "http_client"
     section = "Http"
@@ -501,9 +580,9 @@ class HttpClient(Setting):
     type = int
     default = 2
     desc = """\
-        Set the python parser as default HTTP parser.    
+        Set the python parser as default HTTP parser.
         """
-    
+
     def on_start(self):
         if self.value is not 2:
             v = (self.value,)
@@ -511,7 +590,8 @@ class HttpClient(Setting):
                 v += (1,)
             from pulsar.net import setDefaultClient
             setDefaultClient(v)
-        
+
+
 class HttpParser(Setting):
     name = "http_py_parser"
     section = "Http"
@@ -519,13 +599,13 @@ class HttpParser(Setting):
     action = "store_true"
     default = False
     desc = """\
-    Set the python parser as default HTTP parser.    
+    Set the python parser as default HTTP parser.
         """
-    
+
     def on_start(self):
         if self.value:
-            from pulsar.lib import setDefaultHttpParser, fallback
-            setDefaultHttpParser(fallback.HttpParser)
+            from pulsar.lib import setDefaultHttpParser 
+            setDefaultHttpParser(PyHttpParser)
 
 
 class Debug(Setting):
@@ -537,11 +617,11 @@ class Debug(Setting):
     default = False
     desc = """\
         Turn on debugging in the server.
-        
+
         This limits the number of worker processes to 1 and changes some error
         handling that's sent to clients.
         """
-        
+
 
 class Daemon(Setting):
     name = "daemon"
@@ -552,7 +632,7 @@ class Daemon(Setting):
     default = False
     desc = """\
         Daemonize the Pulsar process.
-        
+
         Detaches the server from the controlling terminal and enters the
         background.
         """
@@ -567,11 +647,20 @@ class Pidfile(Setting):
     default = None
     desc = """\
         A filename to use for the PID file.
-        
+
         If not set, no PID file will be written.
         """
 
 
+class Password(Setting):
+    name = "password"
+    section = "Server Mechanics"
+    flags = ["--password"]
+    validator = validate_string
+    default = None
+    desc = """Set a password for the server"""
+
+
 class User(Setting):
     name = "user"
     section = "Server Mechanics"
@@ -581,12 +670,12 @@ class User(Setting):
     default = None
     desc = """\
         Switch worker processes to run as this user.
-        
+
         A valid user id (as an integer) or the name of a user that can be
         retrieved with a call to pwd.getpwnam(value) or None to not change
         the worker process user.
         """
-        
+
 
 class Group(Setting):
     name = "group"
@@ -597,48 +686,13 @@ class Group(Setting):
     default = None
     desc = """\
         Switch worker process to run as this group.
-        
+
         A valid group id (as an integer) or the name of a user that can be
         retrieved with a call to pwd.getgrnam(value) or None to not change
         the worker processes group.
         """
 
 
-class Umask(Setting):
-    name = "umask"
-    section = "Server Mechanics"
-    flags = ["-m", "--umask"]
-    validator = validate_pos_int
-    type = int
-    default = 0
-    desc = """\
-        A bit mask for the file mode on files written by Gunicorn.
-        
-        Note that this affects unix socket permissions.
-        
-        A valid value for the os.umask(mode) call or a string compatible with
-        int(value, 0) (0 means Python guesses the base, so values like "0",
-        "0xFF", "0022" are valid for decimal, hex, and octal representations)
-        """
-
-
-class TmpUploadDir(Setting):
-    name = "tmp_upload_dir"
-    section = "Server Mechanics"
-    meta = "DIR"
-    validator = validate_string
-    default = None
-    desc = """\
-        Directory to store temporary request data as they are read.
-        
-        This may disappear in the near future.
-        
-        This path should be writable by the process permissions set for Gunicorn
-        workers. If not specified, Gunicorn will choose a system generated
-        temporary directory.
-        """
-
-
 class Loglevel(Setting):
     name = "loglevel"
     section = "Logging"
@@ -647,7 +701,7 @@ class Loglevel(Setting):
     validator = validate_string
     default = "info"
     desc = """The granularity of log outputs.
-        
+
 Valid level names are:
 
  * debug
@@ -665,8 +719,8 @@ class LogEvery(Setting):
     validator = validate_pos_int
     default = 0
     desc = """Log information every n seconds"""
-    
-    
+
+
 class LogConfig(Setting):
     name = "logconfig"
     section = "Logging"
@@ -674,11 +728,11 @@ class LogConfig(Setting):
     validator = validate_dict
     desc = '''
     The logging configuration dictionary.
-    
+
     This settings can only be specified on a config file
     '''
-    
-    
+
+
 class Procname(Setting):
     name = "proc_name"
     section = "Process Naming"
@@ -688,12 +742,12 @@ class Procname(Setting):
     default = None
     desc = """\
         A base to use with setproctitle for process naming.
-        
+
         This affects things like ``ps`` and ``top``. If you're going to be
         running more than one instance of Pulsar you'll probably want to set a
         name to tell them apart. This requires that you install the setproctitle
         module.
-        
+
         It defaults to 'pulsar'.
         """
 
@@ -716,7 +770,7 @@ class WhenReady(Setting):
     default = staticmethod(def_start_server)
     desc = """\
         Called just after the server is started.
-        
+
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
@@ -729,12 +783,12 @@ class Prefork(Setting):
     type = "callable"
     desc = """\
         Called just before a worker is forked.
-        
+
         The callable needs to accept two instance variables for the Arbiter and
         new Worker.
         """
-        
-    
+
+
 class Postfork(Setting):
     name = "post_fork"
     section = "Server Hooks"
@@ -743,7 +797,7 @@ class Postfork(Setting):
     default = staticmethod(default_process)
     desc = """\
         Called just after a worker has been forked.
-        
+
         The callable needs to accept two instance variables for the Arbiter and
         new Worker.
         """
@@ -757,7 +811,7 @@ class PreExec(Setting):
     default = staticmethod(def_pre_exec)
     desc = """\
         Called just before a new master process is forked.
-        
+
         The callable needs to accept a single instance variable for the Arbiter.
         """
 
@@ -770,7 +824,7 @@ class PreRequest(Setting):
     default = staticmethod(def_pre_request)
     desc = """\
         Called just before a worker processes the request.
-        
+
         The callable needs to accept two instance variables for the Worker and
         the Request.
         """
@@ -812,4 +866,16 @@ class WorkerTask(Setting):
     desc = """Called at every event loop by the worker.
 
         The callable needs to accept one variable for the Worker.
+        """
+        
+        
+class ArbiterTask(Setting):
+    name = "arbiter_task"
+    section = "Server Hooks"
+    validator = validate_callable(1)
+    type = "callable"
+    default = staticmethod(def_worker_exit)
+    desc = """Called at every event loop by the arbiter.
+
+        The callable needs to accept one variable for the Arbiter.
         """
\ No newline at end of file
