@@ -5,33 +5,43 @@ import logging
 from multiprocessing import current_process, Lock
 
 if sys.version_info < (2,7):
-    from pulsar.utils.fallbacks.dictconfig import dictConfig
+    from .fallbacks._dictconfig import dictConfig
+    
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
 else:
     from logging.config import dictConfig
+    from logging import NullHandler
     
+from .structures import AttributeDictionary
 
 SERVER_NAME = 'Pulsar'
 NOLOG = 100
 
 
 __all__ = ['SERVER_NAME',
+           'dictConfig',
+           'NullHandler',
            'getLogger',
            'process_global',
            'LogginMixin',
            'Synchronized',
+           'local_method',
+           'local_property',
            'LocalMixin',
            'LogSelf',
            'LogInformation']
 
 
 LOG_LEVELS = {
-        "critical": logging.CRITICAL,
-        "error": logging.ERROR,
-        "warning": logging.WARNING,
-        "info": logging.INFO,
-        "debug": logging.DEBUG,
-        'none': None
-    }
+    "critical": logging.CRITICAL,
+    "error": logging.ERROR,
+    "warning": logging.WARNING,
+    "info": logging.INFO,
+    "debug": logging.DEBUG,
+    'none': None
+}
 
 
 LOGGING_CONFIG = {
@@ -39,20 +49,21 @@ LOGGING_CONFIG = {
     'disable_existing_loggers': False,
     'formatters': {
         'verbose': {
-            'format': '%(asctime)s [p=%(process)s,t=%(thread)s]\
- [%(levelname)s] [%(name)s] %(message)s',
+            'format': '%(asctime)s [p=%(process)s,t=%(thread)s]'
+                      ' [%(levelname)s] [%(name)s] %(message)s',
             'datefmt': '%Y-%m-%d %H:%M:%S'
         },
         'verbose_color': {
             '()': 'pulsar.utils.tools.ColorFormatter',
-            'format': '%(asctime)s [p=%(process)s,t=%(thread)s]\
- [%(levelname)s] [%(name)s] %(message)s',
+            'format': '%(asctime)s [p=%(process)s,t=%(thread)s]'
+                      ' [%(levelname)s] [%(name)s] %(message)s',
             'datefmt': '%Y-%m-%d %H:%M:%S'
         },
         'simple': {
             'format': '%(asctime)s %(levelname)s %(message)s',
             'datefmt': '%Y-%m-%d %H:%M:%S'
         },
+        'message': {'format': '%(message)s'}
     },
     'handlers': {
         'silent': {
@@ -62,6 +73,10 @@ LOGGING_CONFIG = {
             'level': 'DEBUG',
             'class': 'logging.StreamHandler',
             'formatter': 'verbose_color'
+        },
+        'console_message': {
+            'class': 'logging.StreamHandler',
+            'formatter': 'message'
         }
     },
     'filters ': {},
@@ -71,38 +86,53 @@ LOGGING_CONFIG = {
 
 
 def update_config(config, c):
-    for name in ('handlers','formatters','filters','loggers','root'):
+    for name in ('handlers', 'formatters', 'filters', 'loggers', 'root'):
         if name in c:
             config[name].update(c[name])
 
 
 class LocalMixin(object):
-    
+    '''The :class:`LocalMixin` defines "local" attributes which are
+removed when pickling the object'''
     @property
     def local(self):
-        if not hasattr(self,'_local'):
-            self._local = {}
+        if not hasattr(self, '_local'):
+            self._local = AttributeDictionary()
         return self._local
-     
+    
+    def clear_local(self):
+        self.__dict__.pop('_local', None)
+        
     def __getstate__(self):
         '''Remove the local dictionary.'''
         d = self.__dict__.copy()
-        d.pop('_local',None)
+        d.pop('_local', None)
         return d
     
+
+def local_method(f):
+    name = f.__name__
+    def _(self):
+        local = self.local
+        if name not in local:
+            setattr(local, name, f(self))
+        return getattr(local, name)
+    return _
+    return property(_, doc=f.__doc__)
+
+def local_property(f):
+    return property(local_method(f), doc=f.__doc__)
+    
     
 class SynchronizedMixin(object):
-    
-    @property
+    '''A mixin to be used with class:`LocalMixin` class.'''
+    @local_property
     def lock(self):
-        if '_lock' not in self.local:
-            self.local['_lock'] = Lock()
-        return self.local['_lock']
+        return Lock()
     
     @classmethod
     def make(cls, f):
         """Synchronization decorator for Synchronized member functions. """
-    
         def _(self, *args, **kw):
             lock = self.lock
             lock.acquire()
@@ -110,14 +140,12 @@ class SynchronizedMixin(object):
                 return f(self, *args, **kw)
             finally:
                 lock.release()
-                
         _.thread_safe = True
         _.__doc__ = f.__doc__
-        
         return _
     
     
-class Synchronized(LocalMixin,SynchronizedMixin):
+class Synchronized(LocalMixin, SynchronizedMixin):
     pass
 
 
@@ -163,7 +191,7 @@ string representation of an instance.
         return self.logger.name
     
     def _msg(self, msg):
-        return '{0} - {1}'.format(self.instance,msg)
+        return '%s - %s' % (self.instance, msg)
     
     def _handle(self, name):
         func = getattr(self.logger,name)
@@ -182,21 +210,20 @@ class LogginMixin(Synchronized):
     '''A Mixin used throught the library. It provides built in logging object
 and utilities for pickle.'''
     loglevel = None
-    default_logging_level = None
     default_logging_config = None
     _class_code = None
     
-    def setlog(self, log = None, **kwargs):
+    def setlog(self, log=None, **kwargs):
         if not log:
-            name = getattr(self,'_log_name',self.class_code)
+            name = getattr(self, '_log_name', self.class_code)
             log = getLogger(name)
-        self.local['log'] = log
+        self.local.log = log
         self._log_name = log.name
         return log
         
     @property
     def log(self):
-        return self.local.get('log')      
+        return self.local.log      
     
     def __repr__(self):
         return self.class_code
@@ -216,17 +243,17 @@ and utilities for pickle.'''
         self.__dict__ = state
         self.configure_logging()
     
-    def configure_logging(self, config = None):
+    def configure_logging(self, config=None):
         '''Configure logging. This function is invoked every time an
-instance of this class is unserialized (possibly in a different process domain).
-'''
+instance of this class is un-serialised (possibly in a different
+process domain).'''
         logconfig = None
+        # if the logger was not configured, do so.
         if not process_global('_config_logging'):
             logconfig = copy.deepcopy(LOGGING_CONFIG)
             if config:
-                update_config(logconfig,config)
-            process_global('_config_logging',logconfig,True)
-            
+                update_config(logconfig, config)
+            process_global('_config_logging',logconfig, True)
         loglevel = self.loglevel
         if loglevel is None:
             loglevel = logging.NOTSET
@@ -239,25 +266,19 @@ instance of this class is unserialized (possibly in a different process domain).
                     loglevel = logging._levelNames[lv]
                 else:
                     loglevel = logging.NOTSET
-    
         self.loglevel = loglevel
-
         # No loggers configured. This means no logconfig setting
         # parameter was used. Set up the root logger with default
         # loggers 
         if logconfig:
             if not logconfig.get('root'):
                 if loglevel == logging.NOTSET:
-                    logconfig['root'] = {
-                                    'handlers':['silent']
-                                }
+                    handlers = ['silent']
                 else:
-                    logconfig['root'] = {
-                                    'level': logging.getLevelName(loglevel),
-                                    'handlers':['console']
-                                }
+                    handlers = ['console']
+                logconfig['root'] = {'handlers': handlers}
+            logconfig['root']['level'] = logging.getLevelName(loglevel)
             dictConfig(logconfig)
-        
         self.setlog()
         self.log.setLevel(self.loglevel)
             
